/*

 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "data_home.h"
#include "datamgr.h"
#include "utils.h"

enum input_handler_type {
	INPUT_HANDLER_TYPE_EO,
	INPUT_HANDLER_TYPE_DOWN
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *down;
	Evas_Object *scr;
	Eina_List *list;

	struct datamgr *dm;
	struct bar_item *foc;
};

struct bar_item {
	Evas_Object *eo;
	Evas_Object *badge;

	struct datamgr_item *di;
	struct _priv *priv;
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _priv *priv;
	struct bar_item *bi;

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		if (!data) {
			_ERR("Invalid argument");
			return;
		}
		bi = data;
		priv = bi->priv;
		priv->foc = bi;
		/* fallthrough */
	case INPUT_HANDLER_TYPE_DOWN:
		elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
	case INPUT_HANDLER_TYPE_DOWN:
		elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;
	struct bar_item *bi;

	if (!data)
		return;

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		bi = data;
		priv = bi->priv;
		if (!strcmp(ev->keyname, KEY_DOWN)) {
			viewmgr_push_view(VIEW_RECENT);
		} else if (!strcmp(ev->keyname, KEY_ENTER) ||
				!strcmp(ev->keyname, KEY_ENTER_REMOTE)) {
			if (priv->foc->di->action == ITEM_SELECT_ACTION_PUSH)
				elm_object_signal_emit(priv->base, SIG_HIDE,
						SRC_PROG);
			else
				datamgr_select_item(priv->dm, priv->foc->di);
		}
		break;
	case INPUT_HANDLER_TYPE_DOWN:
		priv = data;
		if (!strcmp(ev->keyname, KEY_UP))
			elm_object_focus_set(priv->foc->eo, EINA_TRUE);
		else if (!strcmp(ev->keyname, KEY_ENTER) ||
				!strcmp(ev->keyname, KEY_ENTER_REMOTE))
			viewmgr_push_view(VIEW_RECENT);
		break;
	default:
		return;
	}
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	struct _priv *priv;
	struct bar_item *bi;

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		if (!data)
			return;

		bi = data;
		priv = bi->priv;
		if (priv->foc->di->action == ITEM_SELECT_ACTION_PUSH)
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		else
			datamgr_select_item(priv->dm, priv->foc->di);
		break;
	case INPUT_HANDLER_TYPE_DOWN:
		viewmgr_push_view(VIEW_RECENT);
		break;
	default:
		return;
	}
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_DOWN:
	case INPUT_HANDLER_TYPE_EO:
		if (!elm_object_focus_get(obj))
			elm_object_focus_set(obj, EINA_TRUE);
		break;
	default:
		return;
	}
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_down = _mouse_down,
	.mouse_move = _mouse_move
};

static bool _add_navigations(struct _priv *priv, Evas_Object *base)
{
	Evas_Object *ly;

	ly = utils_add_layout(base, GRP_HOME_DOWN_ARROW, true,
			PART_HOME_DOWN_ARROW);
	if (!ly) {
		_ERR("failed to add layout");
		return false;
	}

	inputmgr_add_callback(ly, INPUT_HANDLER_TYPE_DOWN, &handler, priv);
	priv->down = ly;

	return true;
}

static Evas_Object *_add_notification_badge(Evas_Object *eo, char *noti)
{
	Evas_Object *badge;

	if (!strcmp(noti, "0"))
		return NULL;

	badge = utils_add_icon(eo, IMAGE_BADGE, PART_BAR_ITEM_BADGE);
	if (!badge) {
		_ERR("failed to add badge");
		return NULL;
	}
	elm_object_part_text_set(eo, PART_BAR_ITEM_BADGE_COUNT, noti);
	evas_object_color_set(badge, 238, 67, 49, 255);

	return badge;
}

static struct bar_item *_pack_bar_item(struct _priv *priv, Evas_Object *box,
		struct datamgr_item *di)
{
	struct bar_item *bi;
	Evas_Object *eo, *ic, *focus_ic, *lbl, *focus_lbl, *bg;

	if (!priv || !box || !di) {
		_ERR("Invalid argument");
		return NULL;
	}

	bi = calloc(1, sizeof(*bi));
	if (!bi) {
		_ERR("failed to calloc bar item");
		return NULL;
	}

	eo = utils_add_layout(box, GRP_HOME_ITEM, true, NULL);
	if (!eo) {
		_ERR("failed to add layout");
		free(bi);
		return NULL;
	}

	ic = utils_add_icon(eo, di->icon, PART_BAR_ITEM_ICON);
	if (!ic)
		goto err;

	focus_ic = utils_add_icon(eo, di->focus_icon, PART_BAR_ITEM_ICON_FOCUS);
	if (!focus_ic)
		goto err;

	if (!strcmp(di->icon, di->focus_icon))
		elm_object_signal_emit(eo, SIG_SHOW_MASK, SRC_PROG);

	lbl = utils_add_label(eo, di->title, STYLE_LABEL_TITLE,
			PART_BAR_ITEM_TITLE);
	if (!lbl)
		goto err;

	focus_lbl = utils_add_label(eo, di->title, STYLE_LABEL_TITLE_FOCUS,
			PART_BAR_ITEM_TITLE_FOCUS);
	if (!focus_lbl)
		goto err;

	bg = utils_add_bg(eo, COLOR_DEFAULT_R, COLOR_DEFAULT_G, COLOR_DEFAULT_B,
			COLOR_DEFAULT_A, PART_BAR_ITEM_BG);
	if (!bg)
		goto err;

	if (di->noti)
		bi->badge = _add_notification_badge(eo, di->subtitle);

	inputmgr_add_callback(eo, INPUT_HANDLER_TYPE_EO, &handler, bi);
	elm_box_pack_end(box, eo);
	evas_object_show(eo);

	bi->priv = priv;
	bi->eo = eo;
	bi->di = di;

	return bi;
err:
	_ERR("failed to add home item");
	evas_object_del(eo);
	free(bi);

	return NULL;
}

static bool _add_home_menu(struct _priv *priv, Evas_Object *base)
{
	Eina_List *list, *l;
	Evas_Object *scr, *box;
	struct bar_item *bi;
	struct datamgr_item *di;

	scr = utils_add_scroller(base);
	if (!scr) {
		_ERR("failed to add scroller");
		return false;
	}

	box = utils_add_box(scr, true);
	if (!box) {
		_ERR("failed to add box");
		evas_object_del(scr);
		return false;
	}
	elm_object_content_set(scr, box);
	elm_object_part_content_set(base, PART_HOME_MENU_BAR, scr);
	priv->scr = scr;

	list = datamgr_get_items(priv->dm);
	if (!list) {
		_ERR("failed to load list");
		evas_object_del(scr);
		return false;
	}

	EINA_LIST_FOREACH(list, l, di) {
		bi = _pack_bar_item(priv, box, di);
		if (!bi)
			continue;

		priv->list = eina_list_append(priv->list, bi);
	}

	return true;
}

static void _hide_done(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	struct _priv *priv = data;

	datamgr_select_item(priv->dm, priv->foc->di);
}

static void _show_done(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	struct _priv *priv = data;

	if (!priv->foc)
		priv->foc = eina_list_data_get(priv->list);

	elm_object_focus_set(priv->foc->eo, EINA_TRUE);
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	struct datamgr *dm;
	Evas_Object *base;

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	dm = datamgr_init(datamgr_home_get_dclass(), VIEW_HOME);
	if (!dm) {
		_ERR("failed to initialize datamgr");
		free(priv);
		return NULL;
	}

	base = utils_add_layout(win, GRP_HOME, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		datamgr_fini(dm);
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = dm;

	if (!_add_navigations(priv, base)) {
		_ERR("failed to load home");
		datamgr_fini(dm);
		evas_object_del(base);
		free(priv);
		return NULL;
	}

	viewmgr_set_view_data(VIEW_HOME, priv);
	elm_object_signal_callback_add(base, SIG_HIDE_DONE, SRC_EDJE,
			_hide_done, priv);
	elm_object_signal_callback_add(base, SIG_SHOW_DONE, SRC_EDJE,
			_show_done, priv);

	return base;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_add_home_menu(priv, priv->base);

	evas_object_show(priv->base);
	elm_object_signal_emit(priv->base, SIG_SHOW_NAVIGATION, SRC_PROG);
	elm_object_signal_emit(priv->base, SIG_SHOW, SRC_PROG);
}

static void _unload_home(struct _priv *priv)
{
	struct bar_item *bi;

	EINA_LIST_FREE(priv->list, bi) {
		inputmgr_remove_callback(bi->eo, &handler);
		evas_object_del(bi->eo);
		free(bi);
	}
	evas_object_del(priv->scr);

	priv->foc = NULL;
	priv->list = NULL;
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_unload_home(priv);

	elm_object_signal_emit(priv->base, SIG_HIDE_NAVIGATION, SRC_PROG);
	evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	datamgr_fini(priv->dm);
	inputmgr_remove_callback(priv->base, &handler);
	inputmgr_remove_callback(priv->down, &handler);
	evas_object_del(priv->base);

	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct bar_item *bi;
	Eina_List *list;
	Evas_Object *badge;

	if (!view_data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_NOTI:
		if (!priv->list)
			return;

		EINA_LIST_FOREACH(priv->list, list, bi) {
			if (!bi->di->noti)
				continue;

			badge = _add_notification_badge(bi->eo, data);
			if (!badge && bi->badge) {
				evas_object_del(bi->badge);
				elm_object_part_text_set(bi->eo,
						PART_BAR_ITEM_BADGE_COUNT, "");
			}
			evas_object_show(bi->eo);
			bi->badge = badge;
		}
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_HOME,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_home_get_vclass(void)
{
	return &vclass;
}

